package be.iminds.ilabt.jfed.ssh_terminal_tool.ssh_key_info;

import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.ssh_key_info.AbstractSshKeyInfo;
import be.iminds.ilabt.jfed.ssh_terminal_tool.putty.PuTTYPrivateKeyFile;
import be.iminds.ilabt.jfed.ssh_terminal_tool.putty.PuttyHelper;
import be.iminds.ilabt.jfed.util.JFedUtils;
import be.iminds.ilabt.jfed.util.KeyUtil;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.PermanentFile;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpContentFile;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpFile;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.interfaces.RSAPrivateCrtKey;
import java.security.interfaces.RSAPublicKey;
import java.util.Date;

/**
 * UserSshKeyInfo
 */
public class UserSshKeyInfo extends AbstractSshKeyInfo {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(UserSshKeyInfo.class);

    private GeniUser geniUser;


    public UserSshKeyInfo(GeniUser geniUser) {
        this.geniUser = geniUser;
    }

    @Override
    public TmpFile getPrivateKeyFile() {
        if (geniUser.getPrivateKeyFile() != null) {
            PermanentFile res = new PermanentFile(geniUser.getPrivateKeyFile());
            ensureMatchingPubFile(res);
            return res;
        }

        LOG.debug("geniUser has no privateKeyFile. geniUser class: " + geniUser.getClass().getName());

        return getUnencryptedPrivateKeyFile();
    }

    @Override
    public TmpFile getPuttyKeyFile() {
        String ppkContent = PuttyHelper.getUserEncryptedPPK(geniUser);
        if (ppkContent != null) {
            assert ppkContent.contains("aes256-cbc");

            //we have encrypted key file, so use that.
            TmpFile lockedPuttyFile = new TmpContentFile("sshKeyUsrE", ".ppk", ppkContent);
            LOG.debug("UserSshKeyInfo is using encrypted PPK");
            return lockedPuttyFile;
        } else {
            LOG.debug("UserSshKeyInfo falling back to unencrypted PPK");
            //fall back to unencrypted
            return getUnlockedPuttyKeyFile();
        }
    }

    @Override
    public TmpFile getUnencryptedPrivateKeyFile() {
        File tmpDir = new File(JFedUtils.getUserDataDirectoryFile(), "tmp");
        if (!tmpDir.exists()) tmpDir.mkdir();
        long nr = new Date().getTime();
        String pemKey = new String(KeyUtil.privateKeyToAnyPem(geniUser.getPrivateKey()));
        TmpContentFile unlockedPrivateKeyFile =
                new TmpContentFile("sshKeyUsrUE", ".pem", pemKey, true /*assureUserOnlyPermissions*/);
        ensureMatchingPubFile(unlockedPrivateKeyFile);
        return unlockedPrivateKeyFile;
    }

    @Override
    public TmpFile getUnlockedPuttyKeyFile() {
        File tmpDir = new File(JFedUtils.getUserDataDirectoryFile(), "tmp");
        if (!tmpDir.exists()) tmpDir.mkdir();
        long nr = new Date().getTime();

        String ppkContent =
                PuTTYPrivateKeyFile.createContent("jFed-tmp-ppk", (RSAPublicKey) geniUser.getPublicKey(), (RSAPrivateCrtKey) geniUser.getPrivateKey(), null);

        TmpContentFile unlockedPuttyFile = new TmpContentFile("sskKeyUsrE", ".ppk", ppkContent);

        return unlockedPuttyFile;
    }

//    @Override
//    public void release() {
//        for (File matchingPubKeyfile : matchingPubKeyfiles) {
//            matchingPubKeyfile.delete();
//        }
//        matchingPubKeyfiles.clear();
//    }

    @Override
    public boolean hasUnencryptedPrivateKey() {
        return true;
    }

    @Override
    public PublicKey getPublicKey() {
        if (geniUser != null)
            return geniUser.getPublicKey();
        else return null;
    }

    @Override
    public PrivateKey getPrivateKey() {
        return geniUser.getPrivateKey();
    }
}
