package be.iminds.ilabt.jfed.experimenter_gui;

import be.iminds.ilabt.jfed.experimenter_gui.bugreporting.BugReportDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.dialogs.AboutDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.dialogs.RecoverSliceDialog;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditorTab;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditorTabFactory;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.GTSEditorTabFactory;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.events.ShowGTSEditorEvent;
import be.iminds.ilabt.jfed.experimenter_gui.preferences.PreferencesDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentBindings;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewControllerTab;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.TerminateExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.UpdateExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.OpenRequestRspecTask;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.RecoverSliceTask;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.TasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.BindingsUtil;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ProcessHelper;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonTab;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.ServerType;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.cathive.fx.guice.FXMLComponent;
import com.google.common.eventbus.EventBus;
import javafx.application.HostServices;
import javafx.beans.binding.Bindings;
import javafx.beans.binding.BooleanBinding;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Provider;
import java.io.*;
import java.net.MalformedURLException;
import java.util.List;
import java.util.Optional;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 15:24
 */
@FXMLComponent
public class GeneralRibbonTab extends RibbonTab {
    private static final Logger LOG = LoggerFactory.getLogger(GeneralRibbonTab.class);

    //private final ExperimenterGUI experimenterGUI;

    @FXML
    private RibbonButton newExperimentButton;
    @FXML
    private RibbonButton openExperimentButton;
    @FXML
    private RibbonButton openExperimentURLButton;
    @FXML
    private RibbonButton saveExperimentButton;

    @FXML
    private RibbonButton preferencesButton;

    @FXML
    private RibbonButton startExperimentButton;
    @FXML
    private RibbonButton updateExperimentButton;
    @FXML
    private RibbonButton stopExperimentButton;
    @FXML
    private RibbonButton recoverExperimentButton;
    @FXML
    private RibbonButton openSharedExperimentButton;


    private final GeniUserProvider geniUserProvider;
    private final TasksFactory tasksFactory;
    private final BugReportDialogFactory bugReportDialogFactory;
    private final Provider<RecoverSliceDialog> recoverSliceDialogProvider;
    private final PreferencesDialogFactory preferencesDialogFactory;
    private final AboutDialogFactory aboutDialogFactory;
    private final Provider<HostServices> hostServicesProvider;
    private final TaskService taskService;
    @Named("primaryStage")
    private final Provider<Stage> primaryStageProvider;
    private final JFedGuiPreferences jFedPreferences;
    private final ExperimentEditorTabFactory experimentEditorTabFactory;
    private final GTSEditorTabFactory gtsEditorTabFactory;
    private final EventBus eventBus;

    private final BooleanProperty isDemo =
            new SimpleBooleanProperty(new JFedGuiPreferences().getBoolean(GuiPreferenceKey.PREF_IS_DEMO, false));

    private TabPane tabPane; //registered in registerExperimenterTabPane()

    @Inject
    GeneralRibbonTab(GeniUserProvider geniUserProvider, TasksFactory tasksFactory, BugReportDialogFactory bugReportDialogFactory,
                     Provider<RecoverSliceDialog> recoverSliceDialogProvider, PreferencesDialogFactory preferencesDialogFactory,
                     AboutDialogFactory aboutDialogFactory,
                     Provider<HostServices> hostServicesProvider, TaskService taskService,
                     Provider<Stage> primaryStageProvider, JFedGuiPreferences jFedPreferences,
                     ExperimentEditorTabFactory experimentEditorTabFactory, GTSEditorTabFactory gtsEditorTabFactory,
                     EventBus eventBus) {
        this.geniUserProvider = geniUserProvider;
        this.tasksFactory = tasksFactory;
        this.bugReportDialogFactory = bugReportDialogFactory;
        this.recoverSliceDialogProvider = recoverSliceDialogProvider;
        this.preferencesDialogFactory = preferencesDialogFactory;
        this.aboutDialogFactory = aboutDialogFactory;
        this.hostServicesProvider = hostServicesProvider;
        this.taskService = taskService;
        this.primaryStageProvider = primaryStageProvider;
        this.jFedPreferences = jFedPreferences;
        this.experimentEditorTabFactory = experimentEditorTabFactory;
        this.gtsEditorTabFactory = gtsEditorTabFactory;
        this.eventBus = eventBus;
    }

    @FXML
    private void initialize() {

        //determine if "Open shared" button must be visible
        SfaAuthority userAuth = geniUserProvider.getLoggedInGeniUser().getUserAuthority();
        boolean usersSeeSharedSlices = userAuth.getUrl(ServerType.GeniServerRole.GENI_CH_SA, 2) != null;

        openSharedExperimentButton.setVisible(!usersSeeSharedSlices);
        openSharedExperimentButton.setManaged(!usersSeeSharedSlices);

        preferencesButton.disableProperty().bind(isDemo);

    }

    /**
     * this method must be called by the GUI to bind the experiments TabPane (on the bottom of the GUI) to the buttons
     * of the GeneralRibbonTab.
     * <p>
     * This cannot be done in initialize(), as the FXML of the ExperimenterGUI hasn't loaded at that point
     */
    void registerExperimentsTabPane(TabPane tabPane) {
        this.tabPane = tabPane;

        BooleanBinding isNoExperimentEditorTabSelectedBinding = Bindings.not(
                BindingsUtil.instanceOfBinding(tabPane.getSelectionModel().selectedItemProperty(),
                        ExperimentEditorTab.class)
        );

        saveExperimentButton.disableProperty().bind(isNoExperimentEditorTabSelectedBinding);
        startExperimentButton.disableProperty().bind(isNoExperimentEditorTabSelectedBinding);

        tabPane.getSelectionModel().selectedItemProperty()
                .addListener((observableValue, oldValue, newValue) -> {

                    if (oldValue != null && oldValue instanceof ExperimentViewControllerTab) {
                        updateExperimentButton.disableProperty().unbind();
                        stopExperimentButton.disableProperty().unbind();
                    }

                    if (newValue != null && newValue instanceof ExperimentViewControllerTab) {
                        ExperimentViewController experimentViewController =
                                ((ExperimentViewControllerTab) newValue).getExperimentViewController();

                        BooleanProperty manifestNotAvailable =
                                ExperimentBindings.createManifestNotAvailableBinding(experimentViewController.getExperiment());


                        updateExperimentButton.disableProperty().bind(manifestNotAvailable);
                        stopExperimentButton.disableProperty().bind(
                                Bindings.or(
                                        Bindings.or(
                                                manifestNotAvailable,
                                                experimentViewController.getExperiment().deletingProperty()),
                                        isDemo));
                    }
                });

    }


    @FXML
    public void onNewExperimentAction(ActionEvent actionEvent) {
        ExperimentEditorTab newTab =
                experimentEditorTabFactory.createExperimentEditorTab();
        tabPane.getTabs().add(newTab);
        tabPane.getSelectionModel().select(newTab);
    }

    @FXML
    public void onNewGtsAction(ActionEvent actionEvent) {
        eventBus.post(new ShowGTSEditorEvent());
    }

    @FXML
    private void onOpenExperimentAction() {
        FileChooser fc = new FileChooser();
        fc.setTitle("Open Experiment Definition");
        // bvermeul, FEDIBBTDEV-666, MAC does not understand this it seems, so let's drop it
        //fc.getExtensionFilters().add(FileUtils.RSPEC_EXTENSION_FILTER);
        //fc.getExtensionFilters().add(FileUtils.ALL_EXTENSION_FILTER);

        //set the default open dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));

        if (jFedPreferences.containsPreference(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST)) {
            File dir = new File(jFedPreferences.getString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST));
            if (dir.exists()) {
                fc.setInitialDirectory(dir);
            }
        }


        List<File> files = fc.showOpenMultipleDialog(openExperimentButton.getScene().getWindow());

        if (files == null)
            return;

        //save folder in which files were opened for future file selector dialogs
        jFedPreferences.setString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST,
                files.get(0).getParentFile().getAbsolutePath());

        for (final File file : files) {
            try {
                OpenRequestRspecTask readFileTask =
                        tasksFactory.createOpenRequestRspecTask(openExperimentButton.getScene().getWindow(), file);
                taskService.submitTask(readFileTask);
            } catch (FileNotFoundException e) {
                JFDialogs.create()
                        .owner(primaryStageProvider.get())
                        .title("Opening RSpec failed")
                        .message("Could not open the given file '" + file.getAbsolutePath() + "'")
                        .showException(e);
            }

        }
    }

    @FXML
    public void onOpenExperimentFromURLAction() {
        //create a custom pane for entering the URL

        VBox vbox = new VBox(5);
        Label label = new Label("Please enter the URL:");

        TextField urlTextField = new TextField();
        urlTextField.setPrefColumnCount(40);
        vbox.getChildren().addAll(label, urlTextField);

        Dialog<Optional<String>> dialog = new Dialog<>();
        DialogPane dialogPane = new DialogPane();
        dialogPane.setContent(vbox);
        dialogPane.getButtonTypes().setAll(ButtonType.OK, ButtonType.CANCEL);
        dialogPane.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.CLOUD_DOWNLOAD, Color.CHOCOLATE));

        dialog.initOwner(openExperimentURLButton.getScene().getWindow());
        dialog.setTitle("Open Experiment Definition");
        dialog.setDialogPane(dialogPane);
        dialog.setResultConverter(param -> {
            if (param == ButtonType.OK)
                return Optional.of(urlTextField.getText());
            else
                return Optional.empty();
        });


        dialog.showAndWait().ifPresent(input -> {
            String urlString = input.get();
            try {
                OpenRequestRspecTask openRequestRspecTask =
                        tasksFactory.createOpenRequestRspecTask(openExperimentURLButton.getScene().getWindow(), urlString);
                taskService.submitTask(openRequestRspecTask);
            } catch (MalformedURLException ex) {
                JFDialogs.create()
                        .owner(openExperimentButton)
                        .message("The given URL was malformed.")
                        .masthead("Error while opening Experiment Definition")
                        .title("Error while opening Experiment Definition")
                        .showException(ex);
            } catch (IOException ex) {
                JFDialogs.create()
                        .owner(openExperimentButton)
                        .message("Received an error while opening RSpec.")
                        .masthead("Error while opening Experiment Definition")
                        .title("Error while opening Experiment Definition")
                        .showException(ex);
            }
        });
    }

    @FXML
    private void onRecoverExperimentAction() {
        //show the select slice dialog
        recoverSliceDialogProvider.get().showDialog();
    }


    private ExperimentEditor getActiveExperimentEditor() {
        if (tabPane.getSelectionModel().getSelectedItem() instanceof ExperimentEditorTab) {
            return ((ExperimentEditorTab)
                    tabPane.getSelectionModel().getSelectedItem()).getExperimentEditor();
        } else
            return null;
    }

    @FXML
    private void onSaveExperimentAction() {
        ExperimentEditor activeExperimentEditor = getActiveExperimentEditor();
        assert (activeExperimentEditor != null);

        activeExperimentEditor.save();
    }

    private ExperimentViewController getActiveExperimentViewController() {
        if (tabPane.getSelectionModel().getSelectedItem() instanceof ExperimentViewControllerTab) {
            return ((ExperimentViewControllerTab)
                    tabPane.getSelectionModel().getSelectedItem()).getExperimentViewController();
        } else
            return null;
    }


    @FXML
    private void onUpdateExperimentAction() {
        assert getActiveExperimentViewController() != null;

        eventBus.post(new UpdateExperimentEvent(updateExperimentButton.getScene().getWindow(),
                getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onStopExperimentAction() {
        assert getActiveExperimentViewController() != null;

        eventBus.post(new TerminateExperimentEvent(stopExperimentButton.getScene().getWindow(),
                getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    public void onPreferencesAction() {
        preferencesDialogFactory.showDialog();
    }

    @FXML
    private void onReportBugButtonAction() {
        bugReportDialogFactory.showDialog();
    }

    @FXML
    private void onAboutAction() {
        aboutDialogFactory.showDialog();
    }

    @FXML
    private void onRunExperimentAction(ActionEvent actionEvent) {
        ExperimentEditor activeExperimentEditor = getActiveExperimentEditor();
        assert (activeExperimentEditor != null);
        activeExperimentEditor.startExperiment();
    }

    @FXML
    private void onOpenSharedExperimentAction() {
        //create a custom pane for entering the URL
        try {
            //force the start of a new jFed-window
            ProcessHelper processHelper = new ProcessHelper();
            Process process = processHelper.startNewJavaProcess( Launch.class.toString(), new String[]{
                    "jfed:rspec=http://jfed.iminds.be/ovs.rspec"});

            final BufferedReader bufferedOutputReader = new BufferedReader(new InputStreamReader(process.getInputStream()));

            new Thread(new Runnable() {
                @Override
                public void run() {
                    String line;
                    try {
                        while((line = bufferedOutputReader.readLine()) != null){
                            System.out.println(line);
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }).start();

            final BufferedReader bufferedErrorReader = new BufferedReader(new InputStreamReader(process.getErrorStream()));

            new Thread(new Runnable() {
                @Override
                public void run() {
                    String line;
                    try {
                        while((line = bufferedErrorReader.readLine()) != null){
                            System.out.println(line);
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }).start();



        }catch(IOException ex){
            LOG.error("Error starting process", ex);
        }


        new OpenSharedSliceDialog().showDialog();

    }

    private class OpenSharedSliceDialog extends GridPane {

        private final TextField urnTextField;

        private final SfaAuthority homeAuthority = geniUserProvider.getLoggedInGeniUser().getUserAuthority();

        //private org.controlsfx.dialog.Dialog dialog;

        private Dialog<String> textInputDialog = new Dialog<>();

        public OpenSharedSliceDialog() {
            urnTextField = new TextField();
            urnTextField.setText("urn:publicid:IDN+wall2.ilabt.iminds.be:PROJECT+slice+EXPERIMENT");
            urnTextField.setPrefColumnCount(40);
            add(urnTextField, 0, 1);

//            final String sliceUrn = urnTextField.getText();
//            if (sliceUrn.isEmpty())
//                return;
//
//            dialog = new org.controlsfx.dialog.Dialog(null, "Open shared experiment");
//
//            dialog.setContent(this);
//            dialog.getActions().setAll(new Action("OK", this), org.controlsfx.dialog.Dialog.ACTION_CANCEL);


            textInputDialog.setTitle("Open shared experiment");

            DialogPane dialogPane = new DialogPane();
            dialogPane.setContent(this);
            dialogPane.setHeaderText("Open shared experiment");
            dialogPane.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.GROUP, Color.GREEN));
            dialogPane.getButtonTypes().setAll(ButtonType.OK, ButtonType.CANCEL);


            textInputDialog.setResultConverter(param -> {
                if (param == ButtonType.OK) {
                    return urnTextField.getText();
                } else {
                    return null;
                }
            });

            textInputDialog.setDialogPane(dialogPane);
        }

        public void showDialog() {

            boolean cancelled = false;

            while (!cancelled) {

                Optional<String> input = textInputDialog.showAndWait();

                if (input.isPresent() && input.get() != null) {
                    final String sliceUrn = input.get();

                    if (sliceUrn.isEmpty())
                        continue;

                    if (!GeniUrn.valid(sliceUrn)) {
                        JFDialogs.create()
                                .owner(this)
                                .message("The given URN is malformed.")
                                .masthead("Error while recovering shared experiment")
                                .title("Error while recovering shared experiment")
                                .showError();
                        continue;
                    }

                    GeniUrn inputUrn = GeniUrn.parse(sliceUrn);
                    if (!inputUrn.getEncodedTopLevelAuthority_withoutSubAuth().equals(homeAuthority.getUrn().getEncodedTopLevelAuthority_withoutSubAuth())) {
                        JFDialogs.create()
                                .owner(this)
                                .message(String.format("You can only open experiments that were shared on your own authority '%s'",
                                        homeAuthority.getHrn()))
                                .masthead("Error while recovering shared experiment")
                                .title("Error while recovering shared experiment")
                                .showError();
                        continue;
                    }

                    RecoverSliceTask recoverSliceTask = tasksFactory.createRecoverSliceTask(sliceUrn);
                    taskService.submitTask(recoverSliceTask);
                    return;
                } else {
                    cancelled = true;
                }
            }
        }
    }

    @FXML
    private void onOpenDocumentationAction() {
        hostServicesProvider.get().showDocument("http://doc.ilabt.iminds.be/jfed-documentation/");
    }


}
