package be.iminds.ilabt.jfed.experimenter_gui.canvas.impl;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.ExperimentCanvas;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.*;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 12/2/2014
 * Time: 16:18
 */
@Singleton
public class CanvasNodeFactory {

    private final AuthorityList authorityList;
    private final JFedConfiguration config;

    @Inject
    public CanvasNodeFactory(AuthorityList authorityList, JFedConfiguration config) {
        this.authorityList = authorityList;
        this.config = config;
    }

    public RspecCanvasNode createRspecCanvasNode(FXRspecNode node, ExperimentCanvas experimentCanvas){
        if (node instanceof DelayRspecNode)
            return new DelayRspecCanvasNode((DelayRspecNode) node, experimentCanvas, authorityList, config);
        else
            return new RspecCanvasNode(node, experimentCanvas, authorityList, config);
    }


    public RspecChannelCanvasNode createRspecChannelCanvasNode(FXRspecChannel rspecChannel, ExperimentCanvas experimentCanvas) {
        return new RspecChannelCanvasNode(rspecChannel, experimentCanvas, authorityList, config);
    }

    public RspecGTSNode createRspecGTSNode(FXGeantTestbedType gtt, ExperimentCanvas experimentCanvas) {
        return new RspecGTSNode(gtt, experimentCanvas);

    }

    public AddressPoolCanvasNode createAddressPoolCanvasNode(FXAddressPool addressPool, ExperimentCanvas experimentCanvas){
        return new AddressPoolCanvasNode(addressPool, experimentCanvas, authorityList,config);
    }
}
