package be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs;

import be.iminds.ilabt.jfed.rspec.model.controller.ExperimentCommand;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.fxml.FXML;
import javafx.scene.control.CheckBoxTreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.control.cell.CheckBoxTreeCell;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.stage.Window;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.control.action.Action;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


public class SaveOutputDialog extends GridPane {
    private static final String FXML = "SaveOutputDialog.fxml";
    @FXML
    private TreeView<String> treeView;

    private List<CheckBoxTreeItem<String>> nodesList = new ArrayList<>();
    private Map<CheckBoxTreeItem<String>, ExperimentCommand> commands = new HashMap<>();

    private final Dialog dialog;
    private final DialogAction saveAction;

    private List<ExperimentCommand> selectedCommands = null;


    public SaveOutputDialog(Window parent, List<FXRspecNode> nodes, List<ExperimentCommand> preSelection) {

        //initialize the Scene
        FXMLUtil.injectFXML(this);

        CheckBoxTreeItem<String> rootItem = new CheckBoxTreeItem<>("Your experiment");
        rootItem.setExpanded(true);
        treeView.setRoot(rootItem);
        treeView.setEditable(false);

        treeView.setCellFactory(CheckBoxTreeCell.<String>forTreeView());

        for (FXRspecNode n : nodes) {
            CheckBoxTreeItem<String> cb = new CheckBoxTreeItem<>(n.getClientId());
            for (ExperimentCommand c : n.getCommands()) {
                CheckBoxTreeItem<String> cbc = new CheckBoxTreeItem<>(c.getTag());
                nodesList.add(cbc);
                commands.put(cbc, c);
                cb.getChildren().add(cbc);

                if (preSelection == null || preSelection.contains(c))
                    cbc.setSelected(true);
            }
            rootItem.getChildren().add(cb);
        }
        //initialize dialog


        dialog = new Dialog(parent, "Save results");

        dialog.setContent(this);
        dialog.setMasthead("Select which output you would like to save");
        dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.SAVE, Color.BLUE));

        saveAction =
                new DialogAction("Save", ButtonBar.ButtonType.OK_DONE, false, false, true, event -> onSaveAction());
        dialog.getActions().setAll(saveAction, Dialog.ACTION_CANCEL);
    }

    private void onSaveAction() {
        List<ExperimentCommand> selectedCommands =
                nodesList.stream()
                        .filter(CheckBoxTreeItem::isSelected)
                        .map(commands::get)
                        .collect(Collectors.toList());

        if (selectedCommands.isEmpty()) {
            JFDialogs.create()
                    .message("Please select at least one command to save!").showWarning();
            return;
        }

        dialog.setResult(saveAction);
        dialog.hide();

    }

    public List<ExperimentCommand> showDialog() {
        Action result = dialog.show();

        if (result == saveAction)
            return selectedCommands;
        else
            return null;
    }
}
