package be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation;

import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.parser.RspecCompare;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import javafx.stage.Window;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.control.action.Action;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.dialog.Dialogs;

/**
 * User: twalcari
 * Date: 1/22/2015
 * Time: 17:02
 */
public class CheckIfLosingDataValidator implements RspecValidator {
    private static Action MORE_DETAILS =
            new DialogAction("More Details...", ButtonBar.ButtonType.HELP, false, true, false) {
                {
                    lock();
                }

                public String toString() {
                    return "EditorRibbonTab.MORE_DETAILS";
                }
            };


    @Override
    public RspecValidationResult validateRSpec(RspecCheckReason reason, RequestRspecSource requestRspecSource, Window dialogOwner) {
        //check if we are losing data
        Boolean isLosingData = requestRspecSource.isLosingData(ModelRspecType.FX);

        if (isLosingData == null || isLosingData) {
            //we are losing data! warn the user.

            Action response;
            if (reason.equals(RspecCheckReason.SWITCH_TO_VISUAL_EDITOR) || reason.equals(RspecCheckReason.RUN_EXPERIMENT))
                response =
                        Dialogs.create().owner(dialogOwner)
                                .message(
                                        "The Rspec contains directives that are not supported by the jFed Experimenter toolkit! " +
                                                (reason.equals(RspecCheckReason.SWITCH_TO_VISUAL_EDITOR)
                                                        ? "Changing to the graphical editor will incur data loss. "
                                                        : "") +
                                                "Are you sure you want to continue?")
                                .title("Data loss imminent!")
                                .masthead("Data loss imminent!").actions(Dialog.ACTION_YES, MORE_DETAILS, Dialog.ACTION_NO)
                                .showWarning();

            else { //reason is CHECK_RSPEC or OPENED_FILE
                response =
                        Dialogs.create().owner(dialogOwner)
                                .message(
                                        "The Rspec contains directives that are not supported by the jFed Experimenter toolkit! " +
                                                "Note that because of this, switching to the graphical editor will incur data loss.")
                                .title("Data loss imminent!")
                                .masthead("Data loss imminent!").actions(MORE_DETAILS, Dialog.ACTION_OK)
                                .showWarning();
            }

            if (response == MORE_DETAILS) {
                Dialogs.create().owner(dialogOwner)
                        .message(
                                RspecCompare.differencesListToString(
                                        requestRspecSource.getLosingDataDifferences(ModelRspecType.FX)
                                )
                        )
                        .title("Data loss details").actions(Dialog.ACTION_CLOSE)
                        .showInformation();
                System.out.println("Differences:\n" +
                        RspecCompare.differencesListToString(
                                requestRspecSource.getLosingDataDifferences(ModelRspecType.FX)));
            }

            if (reason.equals(RspecCheckReason.SWITCH_TO_VISUAL_EDITOR) || reason.equals(RspecCheckReason.RUN_EXPERIMENT)) {
                boolean doAbort = response != Dialog.ACTION_YES;
                return new RspecValidationResult(requestRspecSource, doAbort);
            }else{
                return new RspecValidationResult(requestRspecSource, false /* == ignore */);
            }
        } else
            return null;
    }
}
