package be.iminds.ilabt.jfed.experimenter_gui.editor.views;

import be.iminds.ilabt.jfed.experimenter_gui.controller.view.ControllerUI;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.ControllerUIFactory;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs.AddMultipleCommandDialog;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecEditor;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import javafx.beans.property.ReadOnlyStringProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.scene.layout.Region;

/**
 * User: twalcari
 * Date: 5/20/2015
 * Time: 12:18
 */
public class EditableControllerView implements EditableRspecView {


    private final ModelRspecEditor modelRspecEditor;
    private final ControllerUI controllerUi;

    private final StringProperty status = new SimpleStringProperty();

    public EditableControllerView(ModelRspecEditor modelRspecEditor, ControllerUIFactory controllerUIFactory) {
        this.modelRspecEditor = modelRspecEditor;
        this.controllerUi = controllerUIFactory.createControllerUI(modelRspecEditor);
    }


    @Override
    public Region getView() {
        return controllerUi;
    }

    @Override
    public EditorViewType getType() {
        return EditorViewType.CONTROLLER;
    }

    @Override
    public RequestRspecSource getRequestRspecSource() {
        return new RequestRspecSource(controllerUi.getModelRspec());
    }

    @Override
    public void dispose() {
        controllerUi.getController().unregister();
    }

    @Override
    public ReadOnlyStringProperty statusProperty() {
        return status;
    }

    public ControllerUI getControllerUi() {
        return controllerUi;
    }

    public void showAddCommandDialog() {

        AddMultipleCommandDialog.showSimpleAddDialog(controllerUi.getScene().getWindow(),
                getControllerUi().getController(), modelRspecEditor.getModelRspec().getNodes());
    }
}
