package be.iminds.ilabt.jfed.experimenter_gui.editor.views;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.AbstractEditableCanvas;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.CanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.*;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.NewCanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.addresspool_properties.AddressPoolPropertiesDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.editor.bo.NodeDescription;
import be.iminds.ilabt.jfed.experimenter_gui.editor.channel_properties.ChannelPropertiesDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.editor.events.ShowLinkPropertiesEvent;
import be.iminds.ilabt.jfed.experimenter_gui.editor.events.ShowNodePropertiesEvent;
import be.iminds.ilabt.jfed.experimenter_gui.editor.impl.AddressPoolToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.impl.ChannelToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.impl.GeantTestbedToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.impl.NodeDescriptionToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.events.ShowGTSEditorEvent;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.model.RspecNode;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.*;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.google.common.eventbus.EventBus;
import javafx.geometry.Point2D;
import javafx.scene.control.*;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.util.Optional;

/**
 * User: twalcari
 * Date: 11/18/13
 * Time: 5:11 PM
 */
public class EditableExperimentCanvas extends AbstractEditableCanvas {
    private static final Logger LOG = LoggerFactory.getLogger(EditableExperimentCanvas.class);

    private final ModelRspecEditor modelRspecEditor;
    private final AuthorityList authorityList;

    private final ContextMenu nodeContextMenu;
    private final ContextMenu linkContextMenu;
    private final ContextMenu linkInterfaceContextMenu;
    private final ContextMenu channelContextMenu;
    private final ContextMenu gtsContextMenu;
    private final ContextMenu addressPoolContextMenu;


    private final FXRspecFactory rspecFactory = FXRspecFactory.getInstance();
    private final ChannelPropertiesDialogFactory channelPropertiesDialogFactory;
    private final AddressPoolPropertiesDialogFactory addressPoolPropertiesDialogFactory;

    private final EventBus eventBus;

    public EditableExperimentCanvas(DraggedItemProvider draggedItemProvider, final ModelRspecEditor modelRspecEditor,
                                    JFedConfiguration config, CanvasNodeFactory canvasNodeFactory, AuthorityList authorityList,
                                    ChannelPropertiesDialogFactory channelPropertiesDialogFactory, AddressPoolPropertiesDialogFactory addressPoolPropertiesDialogFactory, EventBus eventBus) {
        super(modelRspecEditor.getModelRspec(), config, canvasNodeFactory, draggedItemProvider, authorityList);
        this.modelRspecEditor = modelRspecEditor;
        this.authorityList = authorityList;
        this.channelPropertiesDialogFactory = channelPropertiesDialogFactory;
        this.addressPoolPropertiesDialogFactory = addressPoolPropertiesDialogFactory;
        this.eventBus = eventBus;

        // create context menus
        {
            this.nodeContextMenu = new ContextMenu();

            MenuItem configureItem =
                    new MenuItem("Configure node", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
            configureItem.setOnAction(actionEvent -> {
                assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
                RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

                onConfigureNodePressed(rspecCanvasNode);
            });
            nodeContextMenu.getItems().add(configureItem);

            MenuItem deleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
            deleteItem.setOnAction(actionEvent -> {
                assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
                RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

                onDeleteNodePressed(rspecCanvasNode);
            });
            nodeContextMenu.getItems().add(deleteItem);
        }

        //contextMenu for links
        {
            this.linkContextMenu = new ContextMenu();


            // was disabled as FXRspecLink.LinkSetting is currently not supported in practice, but it seems to work for VM's now
            MenuItem configureLinkItem =
                    new MenuItem("Configure link", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
            configureLinkItem.setOnAction(actionEvent -> {
                RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

                onConfigureLinkPressed(rspecCanvasLink.getRspecLink());
            });
            linkContextMenu.getItems().add(configureLinkItem);

            MenuItem impairLinkItem =
                    new MenuItem("Add impairment bridge to link", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.EXPAND));
            impairLinkItem.setOnAction(actionEvent -> {
                RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

                convertLinkToImpairmentBridge(rspecCanvasLink.getRspecLink());
            });
            linkContextMenu.getItems().add(impairLinkItem);

            MenuItem linkDeleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
            linkDeleteItem.setOnAction(actionEvent -> {
                RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

                onDeleteLinkPressed(rspecCanvasLink);
            });
            linkContextMenu.getItems().add(linkDeleteItem);
        }


        {
            //contextMenu for interfaces
            this.linkInterfaceContextMenu = new ContextMenu();
            MenuItem deleteItem =
                    new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
            deleteItem.setOnAction(actionEvent -> {
                RspecCanvasLink.InterfaceLink ifaceLink = getSelectionProvider().getSelectedInterfaceLink();
                assert (ifaceLink != null);

                onDeleteLinkInterfacePressed(ifaceLink);
            });
            linkInterfaceContextMenu.getItems().add(deleteItem);
        }

        {
            channelContextMenu = new ContextMenu();
            MenuItem configureChannelMenuItem =
                    new MenuItem("Configure channel", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
            configureChannelMenuItem.setOnAction(actionEvent -> {
                RspecChannelCanvasNode rspecChannelCanvasNode = getSelectionProvider().getSelectedChannel();

                onConfigureChannelPressed(rspecChannelCanvasNode.getChannel());
            });
            channelContextMenu.getItems().add(configureChannelMenuItem);

            MenuItem deleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
            deleteItem.setOnAction(actionEvent -> {
                RspecChannelCanvasNode rspecChannelCanvasNode = getSelectionProvider().getSelectedChannel();

                onDeleteChannelPressed(rspecChannelCanvasNode);
            });
            channelContextMenu.getItems().add(deleteItem);
        }

        {
            gtsContextMenu = new ContextMenu();
            MenuItem editGtsMenuItem =
                    new MenuItem("Edit", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
            editGtsMenuItem.setOnAction(actionEvent -> {
                RspecGTSNode rspecGTSNode = getSelectionProvider().getSelectedGTSNode();

                onEditGTSNodePressed(rspecGTSNode);
            });
            gtsContextMenu.getItems().add(editGtsMenuItem);

            MenuItem deleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
            deleteItem.setOnAction(actionEvent -> {
                RspecGTSNode rspecChannelCanvasNode = getSelectionProvider().getSelectedGTSNode();

                onDeleteGTSNodePressed(rspecChannelCanvasNode);
            });
            gtsContextMenu.getItems().add(deleteItem);
        }


        {
            addressPoolContextMenu = new ContextMenu();
            MenuItem configureMenuItem =
                    new MenuItem("Configure address pool", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
            configureMenuItem.setOnAction(actionEvent -> {
                AddressPoolCanvasNode addressPoolCanvasNode = getSelectionProvider().getSelectedAddressPool();

                onConfigureAddressPoolPressed(addressPoolCanvasNode.getAddressPool());
            });
            addressPoolContextMenu.getItems().add(configureMenuItem);

            MenuItem deleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
            deleteItem.setOnAction(actionEvent -> {
                AddressPoolCanvasNode addressPoolCanvasNode = getSelectionProvider().getSelectedAddressPool();

                onDeleteAddressPoolPressed(addressPoolCanvasNode);
            });
            addressPoolContextMenu.getItems().add(deleteItem);
        }
    }

    private void onEditGTSNodePressed(RspecGTSNode rspecGTSNode) {
        eventBus.post(new ShowGTSEditorEvent(rspecGTSNode.getEditableGtsDsl()));
    }

    private void onDeleteGTSNodePressed(RspecGTSNode rspecGTSNode) {
        rspecGTSNode.remove();
    }

    private void onDeleteChannelPressed(RspecChannelCanvasNode rspecChannelCanvasNode) {
        model.deleteChannel(rspecChannelCanvasNode.getChannel());
        if (getSelectionProvider().getSelectedChannel() == rspecChannelCanvasNode) {
            getSelectionProvider().clearAllSelections();
        }
    }

    private void onConfigureChannelPressed(FXRspecChannel channel) {
        channelPropertiesDialogFactory.showChannelPropertiesDialog(getScene().getWindow(), channel, getModelRspec());

    }

    private void onConfigureAddressPoolPressed(FXAddressPool addressPool) {
        addressPoolPropertiesDialogFactory.showAddressPoolPropertiesDialog(getScene().getWindow(), addressPool, getModelRspec());
    }

    private void onDeleteAddressPoolPressed(AddressPoolCanvasNode addressPoolCanvasNode) {
        model.deleteAddressPool(addressPoolCanvasNode.getAddressPool());
        if (getSelectionProvider().getSelectedAddressPool() == addressPoolCanvasNode) {
            getSelectionProvider().clearAllSelections();
        }
    }


    @Override
    protected void deleteSelectedItem() {
        if (getSelectionProvider().getSelectedCanvasNode() != null) {
            CanvasNode canvasNode = getSelectionProvider().getSelectedCanvasNode();
            if (canvasNode instanceof RspecCanvasNode) {
                onDeleteNodePressed((RspecCanvasNode) canvasNode);
            }
        } else if (getSelectionProvider().getSelectedCanvasLink() != null) {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();
            onDeleteLinkPressed(rspecCanvasLink);
        } else if (getSelectionProvider().getSelectedInterfaceLink() != null) {
            onDeleteLinkInterfacePressed(getSelectionProvider().getSelectedInterfaceLink());
        } else if (getSelectionProvider().getSelectedChannel() != null) {
            onDeleteChannelPressed(getSelectionProvider().getSelectedChannel());
        } else if (getSelectionProvider().getSelectedGTSNode() != null) {
            onDeleteGTSNodePressed(getSelectionProvider().getSelectedGTSNode());
        }
    }

    private void convertLinkToImpairmentBridge(FXRspecLink rspecLink) {
        //check if convertion is possible
        if (rspecLink.getInterfaces().size() > 2) {
            JFDialogs.create()
                    .owner(this)
                    .message("An impairment bridge can only be added on a link between two nodes!")
                    .masthead("Unsupported operation")
                    .showWarning();
            return;
        }


        DelayRspecNode delayRspecNode = modelRspecEditor.convertLinkToImpairmentBridge(rspecLink);

        //show configuration-window for impairmentBridgeNode now
        eventBus.post(new ShowNodePropertiesEvent(delayRspecNode, modelRspecEditor, getScene().getWindow()));
    }


    private void onDeleteLinkInterfacePressed(RspecCanvasLink.InterfaceLink ifaceLink) {

        if (ifaceLink.getRspecInterface().getLink().getInterfaces().size() <= 2) {
            model.deleteLink(ifaceLink.getRspecInterface().getLink());
        } else
            ifaceLink.getRspecInterface().delete();

        if (getSelectionProvider().getSelectedInterfaceLink() == ifaceLink) {
            getSelectionProvider().clearAllSelections();
        }
    }

    private void onConfigureNodePressed(RspecCanvasNode rspecCanvasNode) {
        eventBus.post(new ShowNodePropertiesEvent(rspecCanvasNode.getRspecNode(), modelRspecEditor, getScene().getWindow()));
    }

    private void onConfigureLinkPressed(FXRspecLink rspecLink) {
        eventBus.post(new ShowLinkPropertiesEvent(rspecLink, modelRspecEditor, getScene().getWindow()));
    }


    private void onDeleteNodePressed(RspecCanvasNode rspecCanvasNode) {
        model.deleteNode(rspecCanvasNode.getRspecNode());
        if (getSelectionProvider().getSelectedCanvasNode() == rspecCanvasNode) {
            getSelectionProvider().clearAllSelections();
        }

    }

    private void onDeleteLinkPressed(RspecCanvasLink rspecCanvasLink) {
        model.deleteLink(rspecCanvasLink.getRspecLink());
        if (getSelectionProvider().getSelectedCanvasLink() == rspecCanvasLink) {
            getSelectionProvider().clearAllSelections();
        }
    }

    @Override
    protected void canvasItemOnMouseClicked(MouseEvent mouseEvent) {
        super.canvasItemOnMouseClicked(mouseEvent);

        if (mouseEvent.getSource() instanceof RspecCanvasNode) {
            RspecCanvasNode node = (RspecCanvasNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedCanvasNode(node);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureNodePressed(node);

            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                nodeContextMenu.show(node, mouseEvent.getScreenX(), mouseEvent.getScreenY());

            }
        }

        if (mouseEvent.getSource() instanceof RspecCanvasLink.LinkCenter) {
            RspecCanvasLink link =
                    ((RspecCanvasLink.LinkCenter) mouseEvent.getSource()).getRspecCanvasLink();


            getSelectionProvider().setSelectedCanvasLink(link);
            assert (getSelectionProvider().getSelectedCanvasLink() != null);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureLinkPressed(link.getRspecLink());
            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                linkContextMenu.show(getSelectionProvider().getSelectedCanvasLink().getLinkCenter(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }

        if (mouseEvent.getSource() instanceof RspecCanvasLink.InterfaceLink) {
            if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {

                getSelectionProvider().setSelectedInterfaceLink((RspecCanvasLink.InterfaceLink) mouseEvent.getSource());

                //show context menu
                linkInterfaceContextMenu.show(getSelectionProvider().getSelectedInterfaceLink(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }

        if (mouseEvent.getSource() instanceof RspecChannelCanvasNode) {
            final RspecChannelCanvasNode channelCanvasNode = (RspecChannelCanvasNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedChannel(channelCanvasNode);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureChannelPressed(channelCanvasNode.getChannel());
            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                channelContextMenu.show(getSelectionProvider().getSelectedChannel(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }

        if (mouseEvent.getSource() instanceof RspecGTSNode) {
            final RspecGTSNode gtsNode = (RspecGTSNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedGTSNode(gtsNode);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onEditGTSNodePressed(gtsNode);
            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                gtsContextMenu.show(getSelectionProvider().getSelectedGTSNode(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }

        if (mouseEvent.getSource() instanceof AddressPoolCanvasNode) {
            final AddressPoolCanvasNode addressPoolCanvasNode = (AddressPoolCanvasNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedAddressPool(addressPoolCanvasNode);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureAddressPoolPressed(addressPoolCanvasNode.getAddressPool());
            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                addressPoolContextMenu.show(getSelectionProvider().getSelectedAddressPool(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }
    }

    private boolean testForTestbedLinkSupport(FXRspecNode node) {
        if (node.getComponentManagerId() == null) {
            LOG.warn("ComponentManagerId is null for node '{}'", node.getUniqueId());
            return true;
        }

        JFedConfiguration.TestbedDescription testbedDescription =
                config.getTestbedDescription(node.getComponentManagerId());

        if (testbedDescription != null && !testbedDescription.hasLinkSupport()) {
            return false;
        } else
            return true;
    }

    @Override
    protected boolean processNewLinkRequest(NewCanvasLink newCanvasLink) {
        LOG.trace("Received new link request between {} or {} AND {} {}", newCanvasLink.getOriginNode(), newCanvasLink.getOriginLink(),
                newCanvasLink.getTargetNode(), newCanvasLink.getTargetLink());


        if (newCanvasLink.getOriginNode() != null &&
                !processNodeOfNewLinkRequest(newCanvasLink.getOriginNode(), newCanvasLink.isStitchedLink()))
            return false;

        if (newCanvasLink.getTargetNode() != null &&
                !processNodeOfNewLinkRequest(newCanvasLink.getTargetNode(), newCanvasLink.isStitchedLink()))
            return false;

        //warn for stitched link if necessary

        if (newCanvasLink.isStitchedLink()) {
            Optional<ButtonType> reply = JFDialogs.create()
                    .title("Experimental functionality detected.")
                    .message("Stitched links between testbeds are currently experimental. Are you sure you want to continue?")
                    .showYesNo();

            if (!reply.isPresent() || reply.get() != ButtonType.YES) {
                LOG.trace("User cancelled new link because it is stitched");
                return false;
            }
        }

        //delegate the real link-creation to ModelRspecEditor
        LOG.trace("Received link request is OK!");

        if (newCanvasLink.getOriginNode() != null && newCanvasLink.getTargetNode() != null) {
            FXRspecLink newLink = modelRspecEditor.addLink(newCanvasLink.getOriginNode().getRspecNode(),
                    newCanvasLink.getTargetNode().getRspecNode());

            setStatus("Created a new link '" + newLink.getClientId() + "'");

        } else if (newCanvasLink.getOriginNode() != null) {
            assert newCanvasLink.getTargetLink() != null;
            boolean success = modelRspecEditor.addLink(newCanvasLink.getTargetLink().getRspecLink(),
                    newCanvasLink.getOriginNode().getRspecNode());

            if (success) {
                setStatus(String.format("Added node %s to existing link %s",
                        newCanvasLink.getOriginNode().getRspecNode().getClientId(),
                        newCanvasLink.getTargetLink().getRspecLink().getClientId()));
            }

        } else {
            assert newCanvasLink.getOriginLink() != null;
            assert newCanvasLink.getTargetNode() != null;
            boolean success = modelRspecEditor.addLink(newCanvasLink.getOriginLink().getRspecLink(),
                    newCanvasLink.getTargetNode().getRspecNode());

            if (success) {
                setStatus(String.format("Added node %s to existing link %s",
                        newCanvasLink.getTargetNode().getRspecNode().getClientId(),
                        newCanvasLink.getOriginLink().getRspecLink().getClientId()));
            }
        }

        return true;
    }

    private boolean processNodeOfNewLinkRequest(RspecCanvasNode canvasNode, boolean isStitchedLink) {
        if (!testForTestbedLinkSupport(canvasNode.getRspecNode())) {

            String testbedName =
                    authorityList.finder().findByUrn(canvasNode.getRspecNode().getComponentManagerId(),
                            AuthorityFinder.Purpose.STITCHING).getHrn();

            final Alert alert =
                    new Alert(Alert.AlertType.WARNING, String.format("'%s' does not support links.", testbedName));
            alert.setHeaderText("Cannot add link");
            alert.setTitle("Cannot add link");
            alert.show();

            setStatus(String.format("Cannot add link. '%s' does not support links.", testbedName));
            return false;
        }

        if (isStitchedLink && !supportsStitching(canvasNode)) {
            String testbedName =
                    authorityList.finder().findByUrn(canvasNode.getRspecNode().getComponentManagerId(),
                            AuthorityFinder.Purpose.STITCHING).getHrn();

            ButtonType doNotAdd = new ButtonType("Don't Add Link", ButtonBar.ButtonData.NO);
            ButtonType addAnyWay = new ButtonType("Add Link Anyway", ButtonBar.ButtonData.YES);

            final Alert alert =
                    new Alert(Alert.AlertType.CONFIRMATION, String.format("'%s' does not support stitched links. " +
                            "It is best not to add this link. If you know what you are doing you can go further.", testbedName), doNotAdd, addAnyWay);
            alert.setHeaderText("Cannot add link");
            alert.setTitle("Cannot add link");

            //FIXME: workaround for bug on Linux 8u40
            alert.setResizable(true);
            alert.getDialogPane().setPrefSize(480, 200);

            Optional<ButtonType> result = alert.showAndWait();
            if (result.isPresent() && result.get() == addAnyWay) {
                setStatus(String.format("Will add link despite the fact that '%s' does not support stitched links.", testbedName));
                return true;
            }
            assert !result.isPresent() || result.get() == doNotAdd;
            setStatus(String.format("Cannot add link. '%s' does not support stitched links.", testbedName));
            return false;
        }

        return true;
    }


    private boolean supportsStitching(RspecCanvasNode node) {
        SfaAuthority auth =
                authorityList.finder().findByUrn(node.getRspecNode().getComponentManagerId(), AuthorityFinder.Purpose.STITCHING);

        assert auth != null;

        return auth.hasSpecialCase(SfaAuthority.SpecialCase.SUPPORTS_STITCHING);
    }

    protected void addCanvasNodeFromToolboxItem(ToolboxItem item, Point2D location) {
        if (item instanceof NodeDescriptionToolboxItem) {
            NodeDescriptionToolboxItem ndti = (NodeDescriptionToolboxItem) item;
            FXRspecNode rspecNode = modelRspecEditor.addNode(ndti.getNodeDescription(), location);

            if (rspecNode != null) {

                LOG.debug("Added node {} to canvas", rspecNode.getClientId());
                setStatus("Added node " + rspecNode.getClientId() + " to the canvas");

            }else{
                setStatus("Node was not properly configured!");
            }

        } else if (item instanceof ChannelToolboxItem) {
            ChannelToolboxItem cti = (ChannelToolboxItem) item;

            FXRspecChannel channel = modelRspecEditor.addChannel(location);
            boolean success =
                    channelPropertiesDialogFactory.showChannelPropertiesDialog(getScene().getWindow(), channel, getModelRspec());

            if (success) {
                LOG.debug("Adding channel {} to canvas", channel.getComponentName());
                setStatus("Added channel " + channel.getComponentName() + " to the canvas");

            } else {
                LOG.warn("User did not properly configure channel. Removing it again!");

                getModelRspec().deleteChannel(channel);
                setStatus("Channel was not properly configured!");
            }
        } else if (item instanceof GeantTestbedToolboxItem) {
            FXGeantTestbedType gtt = modelRspecEditor.addGeantTestbedType(location);

        } else if (item instanceof AddressPoolToolboxItem) {
            FXAddressPool addressPool = modelRspecEditor.addAddressPool(location);

            //try to set testbed automatically
            JFedConfiguration.TestbedDescription testbedWithAddressPool = getAddressPoolEnabledTestbedDescription();

            if (testbedWithAddressPool != null) {
                addressPool.setComponentManagerId(testbedWithAddressPool.getUrn());
            } else {
                boolean success =
                        addressPoolPropertiesDialogFactory.showAddressPoolPropertiesDialog(getScene().getWindow(), addressPool, getModelRspec());

                if (success) {
                    LOG.debug("Adding address pool {} to canvas", addressPool.getClientId());
                    setStatus("Added channel " + addressPool.getClientId() + " to the canvas");

                } else {
                    LOG.warn("User did not properly configure address pool. Removing it again!");

                    getModelRspec().deleteAddressPool(addressPool);
                    setStatus("Address pool was not properly configured!");
                }
            }


        } else {
            LOG.error("Could not instantiate CanvasNode from ToolboxItem '{}'", item);
            throw new RuntimeException("Unexpected dragged item type: " + item.getClass().getName());
        }
    }

    @Nullable
    private JFedConfiguration.TestbedDescription getAddressPoolEnabledTestbedDescription() {
        for (NodeDescription nodeDescription : config.getNodeDescriptions()) {
            GeniUrn componentManagerUrn = !nodeDescription.getRspecNodeDescriptions().isEmpty()
                    ? GeniUrn.parse(nodeDescription.getRspecNodeDescriptions().get(0).getComponentManagerUrn())
                    : null;

            if (componentManagerUrn != null) {
                JFedConfiguration.TestbedDescription testbedDescription =
                        config.getTestbedDescription(componentManagerUrn);

                if (testbedDescription != null && testbedDescription.hasAddressPoolSupport())
                    return testbedDescription;
            }
        }
        return null;
    }

    @Override
    protected RspecCanvasNode addCopyOfNodeToCanvas(RspecCanvasNode originalRspecCanvasNode) {
        FXRspecNode newRspecNode =
                rspecFactory.copyNode(model, originalRspecCanvasNode.getRspecNode(), RspecNode.InterfaceCopyMethod.NO_COPY);
        modelRspecEditor.updateNodeClientId(newRspecNode, model.nextNodeName());

        newRspecNode.setEditorX(newRspecNode.getEditorX() + EDITOR_LOCATION_SHIFT);
        newRspecNode.setEditorY(newRspecNode.getEditorY() + EDITOR_LOCATION_SHIFT);

        model.getNodes().add(newRspecNode);

        // Now also copy the interface of the orignal node.
        // We do this using the ModelRspecEditor to assure that IP-addresses etc. are correctly changed.

        for (FXRspecInterface originalIface : originalRspecCanvasNode.getRspecNode().getInterfaces()) {
            if (originalIface.isLinkBound()) {
                LOG.info("Duplicating interface {} of {} to {} into new node {}",
                        originalIface.getUniqueId(),
                        originalRspecCanvasNode.getRspecNode().getUniqueId(),
                        originalIface.getLink().getUniqueId(),
                        newRspecNode.getUniqueId());
                modelRspecEditor.addLink(originalIface.getLink(), newRspecNode);
            } else {
                LOG.info("Duplicating unbound interface {} of {} into new node {}",
                        originalIface.getUniqueId(),
                        originalRspecCanvasNode.getRspecNode().getUniqueId(),
                        newRspecNode.getUniqueId());
                modelRspecEditor.addUnboundInterface(newRspecNode);
            }
        }

        return getRspecCanvasNode(newRspecNode);
    }

}
