package be.iminds.ilabt.jfed.experimenter_gui.slice.actions;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.ExperimentTasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.util.ExecuteOnNotNull;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.stage.Window;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.regex.Pattern;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 15:05
 */
public class CreateDiskImageAction extends ExperimentAction {
    private static final Logger LOG = LoggerFactory.getLogger(CreateDiskImageAction.class);

    private static final Pattern DISK_IMAGE_PATTERN = Pattern.compile("[A-Za-z0-9_-]{1,30}");

    private final FXRspecNode node;
    private final AuthorityList authorityList;


    CreateDiskImageAction(Window parentWindow, ExperimentController experimentController, FXRspecNode node,
                          AuthorityList authorityList, ExperimentTasksFactory experimentTasksFactory) {
        super(parentWindow, experimentController, experimentTasksFactory);
        this.node = node;
        this.authorityList = authorityList;
    }

    @Override
    public void run() {
        assert (Platform.isFxApplicationThread());

        new CreateDiskImageDialog().showDialog();

    }

    private class CreateDiskImageDialog extends GridPane {
        private final org.controlsfx.dialog.Dialog dialog;
        private final DialogAction okAction;

        @FXML
        private TextField textField;
        @FXML
        private CheckBox prepareImageCheckBox;


        CreateDiskImageDialog() {
            FXMLUtil.injectFXML(this);
            dialog = new org.controlsfx.dialog.Dialog(parentWindow, "Create a Disk Image of " + node.getClientId());
            dialog.setMasthead("Create a Disk Image of " + node.getClientId());
            dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.UPLOAD, Color.GREEN));

            dialog.setResizable(false);
            dialog.setContent(this);


            okAction =
                    new DialogAction("OK", org.controlsfx.control.ButtonBar.ButtonType.OK_DONE, false, false, true, this::onOKPressed);

            dialog.getActions().setAll(okAction, org.controlsfx.dialog.Dialog.ACTION_CANCEL);

            new ExecuteOnNotNull<Scene>(this.sceneProperty()) {
                @Override
                public void run() {
                    Platform.runLater(textField::requestFocus);
                }
            };

        }

        private void onOKPressed(ActionEvent actionEvent) {
            if (!DISK_IMAGE_PATTERN.matcher(textField.getText()).matches()) {
                new Alert(Alert.AlertType.ERROR, "Please provide a valid disk image name!").showAndWait();
            } else {
                dialog.hide();

                SfaAuthority auth =
                        authorityList.finder().findByUrn(node.getComponentManagerId(), AuthorityFinder.Purpose.CREATE_SLIVER); //same as createsliver
                if (auth == null) {
                    LOG.error("Could not fetch authority info for node {} with componentManagerId {}",
                            node.getClientId(), node.getComponentManagerId());
                    return;
                }

                boolean global = true; //This was the hardcoded default up to now.

                experimentController.submitExperimentTask(
                        experimentTasksFactory.createCreateDiskImageTask(experimentController, auth, node,
                                getDiskImageName(), global, isPrepareImage()));

            }

        }

        public String getDiskImageName() {
            return textField.getText();
        }

        public boolean isPrepareImage() {
            return prepareImageCheckBox.isSelected();
        }


        public void showDialog() {
            dialog.show();
        }

    }


}
