package be.iminds.ilabt.jfed.experimenter_gui.slice.actions;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.ExperimentTasksFactory;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.stage.Window;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 3/26/2015
 * Time: 10:15
 */
public class ShareLanAction extends ExperimentAction {
    private static final Logger LOG = LoggerFactory.getLogger(ShareLanAction.class);


    private final FXRspecLink link;

    public ShareLanAction(Window parentWindow, ExperimentController experimentController, ExperimentTasksFactory experimentTasksFactory, FXRspecLink link) {
        super(parentWindow, experimentController, experimentTasksFactory);
        this.link = link;
    }

    @Override
    public void run() {
        new SharedLanDialog().showDialog();
    }

    private class SharedLanDialog extends GridPane {
        private final Dialog dialog;

        @FXML
        private TextField linkClientIdField;
        @FXML
        private TextField sharedLanNameField;

        private final DialogAction shareAction;
        private final DialogAction unshareAction;


        SharedLanDialog() {
            FXMLUtil.injectFXML(this);

            shareAction = new DialogAction("Share LAN", ButtonBar.ButtonType.OK_DONE,false,false,true, this::onShareLanAction);
            unshareAction = new DialogAction("Unshare LAN", ButtonBar.ButtonType.OK_DONE,false,false,false, this::onUnshareLanAction);

            linkClientIdField.setText(link.getClientId());
            sharedLanNameField.setText("shared_lan_" + link.getClientId());

            dialog = new Dialog(parentWindow, "Share/Unshare LAN" + link.getClientId());
            dialog.setMasthead("Share/Unshare LAN " + link.getClientId());
            dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.SHARE_ALT, Color.GREEN));

            dialog.setResizable(false);
            dialog.setContent(this);

            dialog.getActions().setAll(shareAction, unshareAction, Dialog.ACTION_CANCEL);

        }


        public void showDialog() {
            dialog.show();
        }

        private boolean checkInput() {
            sharedLanNameField.setText(sharedLanNameField.getText().trim());

            String sharedLanName = sharedLanNameField.getText();

            if (sharedLanName.isEmpty() || !sharedLanName.matches("[\\w-]*")) {
                JFDialogs.create()
                        .owner(this)
                        .message("The shared LAN name is invalid. Allowed characters: [A-Za-z0-9_-]*")
                        .masthead("The entered shared LAN name is invalid.")
                        .showError();
                return false;
            }

            return true;
        }


        @FXML
        public void onUnshareLanAction(ActionEvent actionEvent) {
            if (!checkInput())
                return;

            dialog.hide();

            String sharedLanName = sharedLanNameField.getText();
            experimentController.submitExperimentTask(experimentTasksFactory.createShareLanTask(experimentController, link, sharedLanName, false /*unshare*/));

        }

        @FXML
        public void onShareLanAction(ActionEvent actionEvent) {
            if (!checkInput())
                return;

            dialog.hide();

            String sharedLanName = sharedLanNameField.getText();
            experimentController.submitExperimentTask(experimentTasksFactory.createShareLanTask(experimentController, link, sharedLanName, true /*share*/));
        }
    }

}
