package be.iminds.ilabt.jfed.experimenter_gui.slice.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentBindings;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.EditSshKeysEvent;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.RebootExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.ShareExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import com.google.common.eventbus.EventBus;
import javafx.beans.binding.Bindings;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.fxml.FXML;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 17:20
 */
public class AdvancedSliceRibbonComponentGroupController {
    private final SliceRibbonTab sliceRibbonTab;
    private final EventBus eventBus;

    @FXML
    private RibbonButton restartExperimentButton;
    @FXML
    private RibbonButton editSshKeysExperimentButton;
    @FXML
    private RibbonButton shareSliceButton;

    private final BooleanProperty isDemo =
            new SimpleBooleanProperty(new JFedGuiPreferences().getBoolean(GuiPreferenceKey.PREF_IS_DEMO, false));

    public AdvancedSliceRibbonComponentGroupController(SliceRibbonTab sliceRibbonTab, EventBus eventBus) {
        this.sliceRibbonTab = sliceRibbonTab;
        this.eventBus = eventBus;

        sliceRibbonTab.activeExperimentViewControllerProperty().addListener((observableValue, oldValue, newValue) -> {
            if (oldValue != null) {
                restartExperimentButton.disableProperty().unbind();
                editSshKeysExperimentButton.disableProperty().unbind();
                shareSliceButton.disableProperty().unbind();
            }

            if (newValue != null) {
                BooleanProperty manifestNotAvailable =
                        ExperimentBindings.createManifestNotAvailableBinding(newValue.getExperiment());

                restartExperimentButton.disableProperty().bind(Bindings.or(manifestNotAvailable, isDemo));
                editSshKeysExperimentButton.disableProperty().bind(Bindings.or(manifestNotAvailable, isDemo));
                shareSliceButton.disableProperty().bind(Bindings.or(manifestNotAvailable, isDemo));
            }

        });
    }


    @FXML
    private void onRebootExperimentAction() {
        eventBus.post(new RebootExperimentEvent(restartExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onEditSshKeysExperimentAction() {
        eventBus.post(new EditSshKeysEvent(editSshKeysExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onShareSliceAction() {
        eventBus.post(new ShareExperimentEvent(shareSliceButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }
}
