package be.iminds.ilabt.jfed.experimenter_gui.slice.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.slice.events.RenewExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.TerminateExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.UpdateExperimentEvent;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import com.google.common.eventbus.EventBus;
import javafx.beans.binding.Bindings;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.fxml.FXML;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 17:20
 */
public class GenericSliceRibbonComponentGroupController {
    private final SliceRibbonTab sliceRibbonTab;

    @FXML
    private RibbonButton updateExperimentButton;
    @FXML
    private RibbonButton renewSliceButton;
    @FXML
    private RibbonButton stopExperimentButton;

    private final EventBus eventBus;

    private final BooleanProperty isDemo =
            new SimpleBooleanProperty(new JFedGuiPreferences().getBoolean(GuiPreferenceKey.PREF_IS_DEMO,false));

    public GenericSliceRibbonComponentGroupController(SliceRibbonTab sliceRibbonTab, EventBus eventBus) {
        this.sliceRibbonTab = sliceRibbonTab;
        this.eventBus = eventBus;

        sliceRibbonTab.activeExperimentViewControllerProperty().addListener((observableValue, oldValue, newValue) -> {
            if (oldValue != null) {
                updateExperimentButton.disableProperty().unbind();
                renewSliceButton.disableProperty().unbind();
                stopExperimentButton.disableProperty().unbind();
            }

            if (newValue != null) {
                updateExperimentButton.disableProperty().bind(newValue.getExperiment().sliceProperty().isNull());
                renewSliceButton.disableProperty().bind(
                        Bindings.or(Bindings.or(
                                        newValue.getExperiment().sliceProperty().isNull(),
                                        newValue.getExperiment().deletingProperty()),
                                isDemo)
                );
                stopExperimentButton.disableProperty().bind(
                        Bindings.or(
                                Bindings.or(
                                        newValue.getExperiment().sliceProperty().isNull(),
                                        newValue.getExperiment().deletingProperty()),
                                isDemo)
                );
            }
        });
    }

    @FXML
    public void onUpdateExperimentAction() {
        eventBus.post(new UpdateExperimentEvent(updateExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    public void onStopExperimentAction() {
        eventBus.post(new TerminateExperimentEvent(updateExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }


    @FXML
    private void onRenewSliceAction() {
        eventBus.post(new RenewExperimentEvent(updateExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }
}
