package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.NextExperimentExpiration;
import be.iminds.ilabt.jfed.experimenter_gui.slice.SliceState;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.Instant;
import java.util.concurrent.CountDownLatch;

/**
 * User: twalcari
 * Date: 2/6/2015
 * Time: 09:02
 */
public class CheckExpirationTaskGenerator extends ExperimentTaskGenerator {
    private static final Logger LOG = LoggerFactory.getLogger(CheckExpirationTaskGenerator.class);

    private final HighLevelController hlc;

    public CheckExpirationTaskGenerator(ExperimentController experimentController, HighLevelController hlc) {
        super(experimentController);
        this.hlc = hlc;
    }

    @Override
    public TaskStatusIndicator.Status generateExperimentTasks() {
        assert experiment.getSlice().getExpirationDate() != null;

        checkIfRenewedOrResourcesExpired();

        NextExperimentExpiration nextExperimentExpiration = new NextExperimentExpiration(experiment.getSlice());

        if (experiment.getSliceState() == SliceState.EXPIRING) {

            if (nextExperimentExpiration.getFirstExpirationTime().isBefore(Instant.now())) {
                experiment.setSliceState(SliceState.EXPIRED_OR_EMPTY);
            } else {
                experiment.setSliceState(SliceState.READY);
            }
        } else {
            LOG.warn("Expected experiment to be in state EXPIRING, but it is in state {}", experiment.getSliceState());
        }

        return TaskStatusIndicator.Status.SUCCESS;
    }


    private void checkIfRenewedOrResourcesExpired() {

        assert experiment.getSlice().getExpirationDate() != null;

        //first check if expiration time has changed without we knowing it.
        CountDownLatch updateLatch = new CountDownLatch(1);

        final UpdateExperimentTask updateExperimentTask = new UpdateExperimentTask(experiment, hlc);
        updateExperimentTask.addTaskFinishedCallback(status -> updateLatch.countDown());

        experimentController.submitExperimentTask(updateExperimentTask);

        try {
            updateLatch.await();
        } catch (InterruptedException ignored) {
        }
    }

//
//        private boolean checkIfRenewedOrResourcesExpiredOld() {
//        assert experiment.getSlice().getExpirationDate() != null;
//
//        //figure out if it is the whole slice which is expiring, or one (or more) of the slivers
//        List<Sliver> sliversExpiringNowAndBeforeSlice = null;
//        if (experiment.getSlice().getExpirationDate().isAfter(Instant.now())) {
//            sliversExpiringNowAndBeforeSlice =
//                    experiment.getSlice().getSlivers().stream()
//                            .filter(sliver -> sliver.getExpirationDate() != null &&
//                                    sliver.getExpirationDate().isBefore(Instant.now()) &&
//                                    sliver.getExpirationDate().isBefore(experiment.getSlice().getExpirationDate()))
//                            .collect(Collectors.toList());
//        }
//
//        //first check if expiration time has changed without we knowing it.
//        CountDownLatch updateLatch = new CountDownLatch(1);
//
//        final UpdateExperimentTask updateExperimentTask = new UpdateExperimentTask(experiment, hlc);
//        updateExperimentTask.addTaskFinishedCallback(status -> updateLatch.countDown());
//
//        experimentController.submitExperimentTask(updateExperimentTask);
//
//        try {
//            updateLatch.await();
//        } catch (InterruptedException ignored) {
//        }
//
//
//        List<SfaAuthority> authoritiesToContact = experiment.getSlice().getSlivers().stream()
//                .filter(sliver -> sliver.getExpirationDate() == null || sliver.getExpirationDate().isBefore(Instant.now()))
//                .map(Sliver::getAuthority)
//                .distinct()
//                .filter(SfaAuthority::isReal)
//                .collect(Collectors.toList());
//
//        CountDownLatch authoritiesContactedLatch = new CountDownLatch(authoritiesToContact.size());
//
//        for (SfaAuthority authority : authoritiesToContact) {
//            CheckExpirationTask checkExpirationTask = new CheckExpirationTask(experiment, authority, hlc);
//
//            checkExpirationTask.addTaskFinishedCallback(callback -> authoritiesContactedLatch.countDown());
//        }
//
//        try {
//            authoritiesContactedLatch.await();
//        } catch (InterruptedException ignored) {
//        }
//
//        if (sliversExpiringNowAndBeforeSlice == null) {
//            //check if the slice was renewed, or if ALL resources are expired (e.g. UNALLOCATED)
//
//            if (experiment.getSlice().getExpirationDate().isAfter(Instant.now()))
//                return true;
//
//            boolean allResourcesExpired = true;
//
//            for (Sliver sliver : experiment.getSlice().getSlivers()) {
//                if (sliver.getStatus().getGlobalStatus() != StatusDetails.SliverStatus.UNALLOCATED &&
//                        sliver.getStatus().getGlobalStatus() != StatusDetails.SliverStatus.FAIL)
//                    allResourcesExpired = false;
//            }
//
//            return allResourcesExpired;
//        } else {
//            //we only have to check the slivers which were expiring at the start of the call
//
//            for (Sliver sliver : sliversExpiringNowAndBeforeSlice) {
//                assert (sliver.getExpirationDate() != null);
//                if (sliver.getExpirationDate().isBefore(Instant.now()) &&
//                        sliver.getStatus().getGlobalStatus() != StatusDetails.SliverStatus.UNALLOCATED &&
//                        sliver.getStatus().getGlobalStatus() != StatusDetails.SliverStatus.FAIL)
//                    return false;
//            }
//
//            return true;
//        }
//    }
}
