package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.SliceState;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 17:24
 */
public class StopExperimentAtAuthorityTask extends SingleCallExperimentTask {

    private final SfaAuthority authority;

    StopExperimentAtAuthorityTask(Experiment experiment, SfaAuthority authority, HighLevelController hlc) {
        super(experiment, hlc);
        this.authority = authority;
        assert (experiment.getSlice() != null);

        updateTitle("Release all resources on " + authority.getHrn() + " of experiment '" + experiment.getName() + "'");
    }


    @Override
    public Task createTask() {
        return hlc.deleteSliversAtAuthorityTask(experiment.getSlice(), authority);
    }


//    @Override
//    public void onTaskStarting() {
//        Platform.runLater(() -> {
//            experiment.setDeleting(true);
//        });
//    }
//
//
    @Override
    public void onTaskFinished(TaskExecution taskExecution) {
        boolean hasActiveResources = false;

        for (StatusDetails.SliverStatus sliverStatus : experiment.getSlice().getStatus().getStatusBySliverUrn().values()) {
            if(sliverStatus == StatusDetails.SliverStatus.READY || sliverStatus == StatusDetails.SliverStatus.CHANGING)
                hasActiveResources = true;
        }

        for (StatusDetails.SliverStatus sliverStatus : experiment.getSlice().getStatus().getStatusByComponentUrn().values()) {
            if(sliverStatus == StatusDetails.SliverStatus.READY || sliverStatus == StatusDetails.SliverStatus.CHANGING)
                hasActiveResources = true;
        }

        if(!hasActiveResources){
            experiment.setSliceState(SliceState.EXPIRED_OR_EMPTY);
        }

    }
}
