package be.iminds.ilabt.jfed.experimenter_gui.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import javafx.concurrent.Task;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 3/18/2015
 * Time: 11:39
 */
public class FetchAllSlicesInfoTask extends Task<Void> {

    private final EasyModel easyModel;
    private final HighLevelController hlc;

    public FetchAllSlicesInfoTask(EasyModel easyModel, HighLevelController hlc) {
        this.easyModel = easyModel;

        this.hlc = hlc;
    }

    @Override
    protected Void call() throws Exception {
        updateMessage("Fetching all existing experiments");

        final HighLevelController.ListSlicesTask refreshSlicesTask = hlc.getSlicesForUser();

        TaskExecution refreshTaskExecution = hlc.submitTask(refreshSlicesTask);

        while (!refreshTaskExecution.isCompleted()) {
            Thread.sleep(100);
        }

        updateMessage(String.format("Received %d experiments", refreshSlicesTask.getSlices().size()));

        List<be.iminds.ilabt.jfed.highlevel.controller.Task> extraTasks =
                easyModel.getSlices().stream()
                        .filter(slice -> slice.getExpirationDate() == null)
                        .map(hlc::getAggregatesForSlice).collect(Collectors.toList());

        if (!extraTasks.isEmpty()) {

            updateMessage(String.format("Received %d experiments. Requesting extra info.", refreshSlicesTask.getSlices().size()));

            CountDownLatch latch = new CountDownLatch(extraTasks.size());

            for (be.iminds.ilabt.jfed.highlevel.controller.Task extraTask : extraTasks) {
                hlc.submitTask(extraTask, (task1, taskExecution, state) -> {
                    latch.countDown();
                    updateMessage(String.format("Received %d experiments. Requesting extra info (%d left).",
                            refreshSlicesTask.getSlices().size(), latch.getCount()));
                });
            }
            latch.await();
        }
        return null;
    }
}
