package be.iminds.ilabt.jfed.experimenter_gui.ui;

import be.iminds.ilabt.jfed.experimenter_gui.editor.bo.NodeDescription;
import be.iminds.ilabt.jfed.experimenter_gui.health.HealthStatus;
import be.iminds.ilabt.jfed.experimenter_gui.health.MaintenanceMessage;
import be.iminds.ilabt.jfed.experimenter_gui.util.DateTimeUtils;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.Glyph;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

/**
 * User: twalcari
 * Date: 9/16/2015
 * Time: 14:52
 */
public class HealthStatusPane extends StackPane {
    private static final Comparator<? super MaintenanceMessage> MAINTENANCE_MESSAGE_COMPARATOR =
            Comparator.nullsLast(Comparator.comparing(MaintenanceMessage::getStartTime))
                    .thenComparing(Comparator.nullsLast(Comparator.comparing(MaintenanceMessage::getEndTime)))
                    .thenComparing(MaintenanceMessage::getMessage);

    @FXML
    private Label noInfoLabel;
    @FXML
    private GridPane infoGridPane;

    @FXML
    private Label healthKeyLabel;
    @FXML
    private Label healthLabel;
//    @FXML
//    private Label reputationLabel;
    @FXML
    private Label freeLabel;
    @FXML
    private Label freeInfoLabel;
    @FXML
    private Label ipFreeLabel;
    @FXML
    private Label ipFreeInfoLabel;
    @FXML
    private Label maintenanceLabel;

    @FXML
    private Glyph healthGlyph;
//    @FXML
//    private Glyph reputationGlyph;
    @FXML
    private AvailabilityMeter availabiltyMeter;
    @FXML
    private AvailabilityMeter ipAvailabiltyMeter;
    @FXML
    private Glyph maintenanceGlyph;

    @FXML
    private Button maintenanceInfoButton;

    private final ObjectProperty<NodeDescription.NodeType> type = new SimpleObjectProperty<>(null);
    private final ObjectProperty<HealthStatus> healthStatus = new SimpleObjectProperty<>(null);

    public HealthStatusPane() {
        FXMLUtil.injectFXML(this);

        availabiltyMeter.useVmCalculationMethodProperty().bind(type.isEqualTo(NodeDescription.NodeType.VM));
    }


    public void setHealthStatus(@Nullable HealthStatus status) {
        healthStatus.setValue(status);

        if (status == null) {
            noInfoLabel.setVisible(true);
            infoGridPane.setVisible(false);
            return;
        }
        noInfoLabel.setVisible(false);
        infoGridPane.setVisible(true);

        Tooltip healthTooltip;
        if (status.getHealthExplanationFull() != null)
            healthTooltip = new Tooltip(status.getHealthExplanationFull());
        else
            healthTooltip = new Tooltip("Health explanation unknown");

        healthLabel.setText(status.getHealth() != null ? status.getHealth() + "%" : "-");
        setGlyphColor(healthGlyph, status.getHealth());

        healthLabel.setTooltip(healthTooltip);
        healthKeyLabel.setTooltip(healthTooltip);
        healthGlyph.setTooltip(healthTooltip);

//        reputationLabel.setText(status.getReputation() != null ? status.getReputation() + "%" : "-");
//        setGlyphColor(reputationGlyph, status.getReputation());

        if (NodeDescription.NodeType.RAW_PC.equals(type.get())
                && status.getRawPCsAvailable() != null
                && status.getRawPCsTotal() != null
                && status.getRawPCsTotal() != 0) {
            freeLabel.setText("Free " + NodeDescription.NodeType.RAW_PC.getFriendlyName() + ":");

            availabiltyMeter.setAvailable(status.getRawPCsAvailable());
            availabiltyMeter.setTotal(status.getRawPCsTotal());
            availabiltyMeter.setUndetermined(false);

            freeInfoLabel.setText(status.getRawPCsAvailable() + " of " + status.getRawPCsTotal());

        } else if (NodeDescription.NodeType.VM.equals(type.get()) &&
                status.getVMsAvailable() != null
                && status.getVMsTotal() != null
                && status.getVMsTotal() != 0) {
            freeLabel.setText("Free " + NodeDescription.NodeType.VM.getFriendlyName() + ":");
            availabiltyMeter.setAvailable(status.getVMsAvailable());
            availabiltyMeter.setTotal(status.getVMsTotal());
            availabiltyMeter.setUndetermined(false);
            freeInfoLabel.setText(status.getVMsAvailable() + " of " + status.getVMsTotal());
        } else if(status.getAnyAvailable() != null
                && status.getAnyTotal() != null && status.getAnyTotal() != 0) {
            freeLabel.setText("Free resources:");
            availabiltyMeter.setAvailable(status.getAnyAvailable());
            availabiltyMeter.setTotal(status.getAnyTotal());
            availabiltyMeter.setUndetermined(false);
            freeInfoLabel.setText(status.getAnyAvailable() + " of " + status.getAnyTotal());
        }
        else {
            freeLabel.setText("Free resources:");
            availabiltyMeter.setAvailable(0);
            availabiltyMeter.setTotal(0);
            availabiltyMeter.setUndetermined(true);
            freeInfoLabel.setText("-");
        }

        if(status.getIpv4sTotal() != null && status.getIpv4sTotal() > 0){

            ipAvailabiltyMeter.setAvailable(status.getIpv4sAvailable());
            ipAvailabiltyMeter.setTotal(status.getIpv4sTotal());
            ipAvailabiltyMeter.setUndetermined(false);
            ipFreeInfoLabel.setText(status.getIpv4sAvailable() + " of " + status.getIpv4sTotal());
        }   else{
            ipAvailabiltyMeter.setAvailable(0);
            ipAvailabiltyMeter.setTotal(0);
            ipAvailabiltyMeter.setUndetermined(true);
            ipFreeInfoLabel.setText("-");
        }


        Optional<MaintenanceMessage> activeMaintenance;
        Optional<MaintenanceMessage> upcomingMaintenance;
        if (status.getMaintenance() != null && !status.getMaintenance().isEmpty()) {
            activeMaintenance =
                    status.getMaintenance().stream()
                            .filter(mm -> mm.getStartTime() == null || mm.getStartTime().isBefore(ZonedDateTime.now()))
                            .filter(mm -> mm.getEndTime() == null || mm.getEndTime().isAfter(ZonedDateTime.now()))
                            .sorted(MAINTENANCE_MESSAGE_COMPARATOR)
                            .findFirst();

            upcomingMaintenance =
                    status.getMaintenance().stream()
                            .filter(mm -> mm.getStartTime() != null && mm.getStartTime().isAfter(ZonedDateTime.now()))
                            .sorted(MAINTENANCE_MESSAGE_COMPARATOR)
                            .findFirst();
        } else {
            activeMaintenance = Optional.empty();
            upcomingMaintenance = Optional.empty();
        }
        if (activeMaintenance.isPresent()) {
            StringBuilder maintenanceMessage = new StringBuilder("Currently in maintenance ");

            if (activeMaintenance.get().getStartTime() != null)
                maintenanceMessage.append("from ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.get().getStartTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(" ");
            if (activeMaintenance.get().getEndTime() != null)
                maintenanceMessage.append("until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.get().getEndTime().withZoneSameInstant(ZoneId.systemDefault())));

            maintenanceLabel.setText(maintenanceMessage.toString());
            maintenanceGlyph.setTextFill(Color.RED);
        } else if (upcomingMaintenance.isPresent()) {
            StringBuilder maintenanceMessage = new StringBuilder("Upcoming maintenance ");

            if (upcomingMaintenance.get().getStartTime() != null)
                maintenanceMessage.append("from ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(upcomingMaintenance.get().getStartTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(" ");
            if (upcomingMaintenance.get().getEndTime() != null)
                maintenanceMessage.append("until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(upcomingMaintenance.get().getEndTime().withZoneSameInstant(ZoneId.systemDefault())));

            maintenanceLabel.setText(maintenanceMessage.toString());
            maintenanceGlyph.setTextFill(Color.ORANGE);
        } else {
            maintenanceLabel.setText("None planned");
            maintenanceGlyph.setTextFill(Color.GRAY);
        }

        maintenanceInfoButton.setDisable(!activeMaintenance.isPresent() && !upcomingMaintenance.isPresent());
    }

    @FXML
    private void showMaintenanceInfo() {
        assert healthStatus.get() != null;
        assert healthStatus.get().getMaintenance() != null;
        assert !healthStatus.get().getMaintenance().isEmpty();

        List<MaintenanceMessage> maintenances = new ArrayList<>(healthStatus.get().getMaintenance());
        maintenances.sort(MAINTENANCE_MESSAGE_COMPARATOR);

        StringBuilder maintenanceMessage = new StringBuilder();

        for (MaintenanceMessage activeMaintenance : maintenances) {
            if (activeMaintenance.getStartTime() != null && activeMaintenance.getEndTime() != null) {
                maintenanceMessage.append("From ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getStartTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(" until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getEndTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(": ");
            } else if (activeMaintenance.getStartTime() != null) {
                maintenanceMessage.append("From ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getStartTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(": ");
            } else if (activeMaintenance.getEndTime() != null) {
                maintenanceMessage
                        .append("Until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getEndTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(": ");
            }

            maintenanceMessage.append(activeMaintenance.getMessage()).append("\n\n");
        }

        Alert infoAlert = new Alert(Alert.AlertType.INFORMATION);
        infoAlert.setHeaderText("Planned maintenance");
        infoAlert.setContentText(maintenanceMessage.toString().trim());
        infoAlert.showAndWait();


    }

    private static void setGlyphColor(@Nonnull Glyph glyph, @Nullable Integer value) {
        if (value == null) {
            glyph.setTextFill(Color.GRAY);
        } else if (value > 95) {
            glyph.setTextFill(Color.GREEN);
        } else if (value > 66) {
            glyph.setTextFill(Color.YELLOWGREEN);
        } else if (value > 33) {
            glyph.setTextFill(Color.ORANGE);
        } else {
            glyph.setTextFill(Color.RED);
        }
    }

    public NodeDescription.NodeType getType() {
        return type.get();
    }

    public ObjectProperty<NodeDescription.NodeType> typeProperty() {
        return type;
    }

    public void setType(NodeDescription.NodeType type) {
        this.type.set(type);
    }
}
