package be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar;

import be.iminds.ilabt.jfed.experimenter_gui.slice.actions.SaveManifestAction;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.DeleteDiskImageTask;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.RecoverSliceTask;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.beans.InvalidationListener;
import javafx.beans.WeakInvalidationListener;
import javafx.collections.ListChangeListener;
import javafx.concurrent.Task;
import javafx.event.EventHandler;
import javafx.geometry.Point2D;
import javafx.geometry.Pos;
import javafx.scene.control.ProgressBar;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.paint.Color;
import javafx.util.Duration;
import org.controlsfx.control.PopOver;
import org.controlsfx.control.TaskProgressView;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 11/28/2014
 * Time: 11:48
 */
public class TasksProgressIndicator extends HBox {
    private static final Logger LOG = LoggerFactory.getLogger(TasksProgressIndicator.class);

    private final ProgressBar progressBar = new ProgressBar();

    private final PopOver popOver = new PopOver();
    private final TaskProgressView<Task<?>> taskProgressView = new TaskProgressView<>();

    private final InvalidationListener progressInvalidationListener = invalidated -> {
        progressBar.setProgress(taskProgressView.getTasks().stream().mapToDouble(Task::getProgress).average().orElse(0));
    };


    public TasksProgressIndicator() {
        setAlignment(Pos.CENTER);
        setVisible(false);

        progressBar.setProgress(0);

        getChildren().add(progressBar);
        HBox.setHgrow(progressBar, Priority.ALWAYS);

        popOver.setContentNode(taskProgressView);
        popOver.setArrowLocation(PopOver.ArrowLocation.BOTTOM_RIGHT);
        popOver.setAutoHide(true);

        setPrefWidth(100);

        taskProgressView.setGraphicFactory(task -> {
            if (task instanceof RecoverSliceTask) {
                return GlyphUtils.createDialogGlyph(FontAwesome.Glyph.AMBULANCE, Color.GREEN);
            } else if (task instanceof DeleteDiskImageTask) {
                return GlyphUtils.createDialogGlyph(FontAwesome.Glyph.TIMES, Color.RED);
            } else if ( task instanceof SaveManifestAction.SaveManifestTask) {
                return GlyphUtils.createDialogGlyph(FontAwesome.Glyph.SAVE, Color.BLUE);
            } else if (task instanceof SaveManifestAction.SaveCSVTask ) {
                return GlyphUtils.createDialogGlyph(FontAwesome.Glyph.SAVE, Color.ORANGERED);
            } else {
                return null;
            }
        });


        progressBar.setOnMouseEntered(new EventHandler<MouseEvent>() {
            @Override
            public void handle(MouseEvent event) {
                if (!taskProgressView.getTasks().isEmpty()) {
                    Point2D locationOnScreen = localToScreen(getWidth() / 2, 0);
                    popOver.show(TasksProgressIndicator.this, locationOnScreen.getX(), locationOnScreen.getY(), Duration.seconds(0.5));
                }
            }
        });


        taskProgressView.getTasks().addListener(new ListChangeListener<Task<?>>() {
            @Override
            public void onChanged(Change<? extends Task<?>> c) {
                while (c.next()) {
                    if (c.wasAdded()) {
                        for (Task t : c.getAddedSubList()) {
                            t.progressProperty().addListener(new WeakInvalidationListener(progressInvalidationListener));
                        }
                    }
                }
                TasksProgressIndicator.this.setVisible(!taskProgressView.getTasks().isEmpty());
                taskProgressView.setPrefHeight(Math.min(300, taskProgressView.getTasks().size() * 70));
            }
        });
    }


    public void followTaskProgress(Task<?> task) {
        taskProgressView.getTasks().add(task);
    }
}
