package be.iminds.ilabt.jfed.highlevel.model;

import be.iminds.ilabt.jfed.rspec.model.ModelRspec;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.parser.RspecParseException;
import be.iminds.ilabt.jfed.rspec.rspec_source.AdvertisementRspecSource;
import be.iminds.ilabt.jfed.rspec.rspec_source.ManifestRspecSource;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.rspec.rspec_source.RspecSource;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * RSpec
 */
public class RSpecInfo {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(RSpecInfo.class);

    public enum RspecType {REQUEST, REQUEST_ECHO, MANIFEST, ADVERTISEMENT}

    private String stringContent;
    private RspecSource content;
    private RspecType type;
    private Slice slice;
    private List<Sliver> slivers;

    private AuthorityInfo authority;

    public RSpecInfo(String stringContent, RspecType type, Slice slice, List<Sliver> slivers, AuthorityInfo authority) {
        this.stringContent = stringContent;
        this.type = type;
        this.slice = slice;
        this.slivers = slivers;
        this.authority = authority;
        this.content = null;
    }

    public RSpecInfo(String stringContent, RspecType type, Slice slice, List<Sliver> slivers, AuthorityInfo authority, ModelRspec content) {
        this.stringContent = stringContent;
        this.type = type;
        this.slice = slice;
        this.slivers = slivers;
        this.authority = authority;
        switch (type) {
            case REQUEST_ECHO:
            case REQUEST: {
                this.content = new RequestRspecSource(content);
                break;
            }
            case MANIFEST: {
                this.content = new ManifestRspecSource(stringContent, ModelRspecType.FX);
                break;
            }
            case ADVERTISEMENT: {
                this.content = new AdvertisementRspecSource(stringContent,ModelRspecType.FX);
                break;
            }
            default:
                throw new RuntimeException("Unsupported RSpec type: " + type);
        }

    }

    public RSpecInfo(String stringContent, RspecType type, Slice slice, List<Sliver> slivers, AuthorityInfo authority, RspecSource content) {
        this.stringContent = stringContent;
        this.type = type;
        this.slice = slice;
        this.slivers = slivers;
        this.authority = authority;
        this.content = content;
    }

    public String getStringContent() {
        return stringContent;
    }

    public RspecSource getRSpec() throws RspecParseException {
        if (content == null) {
            LOG.debug("Parsing Rspec of length=" + stringContent.length() + "... ");
            switch (type) {
                case REQUEST_ECHO:
                case REQUEST: {
                    content = new RequestRspecSource(stringContent, ModelRspecType.FX);
                    break;
                }
                case MANIFEST: {
                    content = new ManifestRspecSource(stringContent, ModelRspecType.FX);
                    break;
                }
                case ADVERTISEMENT: {
                    content = new AdvertisementRspecSource(stringContent, ModelRspecType.FX);
                    break;
                }
                default:
                    throw new RspecParseException(null, "Unsupported RSpec type: " + type);
            }
        }

        return content;
    }

    public RspecType getType() {
        return type;
    }

    public Slice getSlice() {
        return slice;
    }

    public List<Sliver> getSlivers() {
        return slivers;
    }

    public AuthorityInfo getAuthority() {
        return authority;
    }
}
